#include <stdio.h>
#include <math.h>
#include "../nd.h"

static void InitND (TParametersND *NDparametros){

  printf("-- Init the ND .\n");
  
  NDparametros->geometryRect=0;		

  NDparametros->front=0.85F;
  NDparametros->back=0.235F;
  NDparametros->left=0.335F;

  NDparametros->R=0.2F;

  NDparametros->holonomic=0;

  NDparametros->vlmax=0.3F;
  NDparametros->vamax=0.5F;

  NDparametros->almax=0.75F;
  NDparametros->aamax=0.75F;

  NDparametros->dsmax=0.4F;
  NDparametros->dsmin=NDparametros->dsmax/4.0F;
  NDparametros->enlarge=0.1F;
  
  NDparametros->discontinuity=0.4F;
  NDparametros->T=0.2F;


  // NDparametros->laser=0.72F; // Compatibilidad josuna.

  

}

void main(){

	FILE *fp;

	TParametersND NDparam;
	TVelocities *velo;
	TCoordenadas objetivo;
	TInfoMovimiento movimiento;
	TInfoEntorno entorno;

	float x_Robot_GLobal_Location,y_Robot_GLobal_Location,theta_Robot_GLobal_Location;
	float x_obstacle1,y_obstacle1,x_obstacle2,y_obstacle2,x_obstacle3,y_obstacle3;
	float x_goal,y_goal;

	int i;
	
	// Debugg
	fp = fopen("coordenadas.out","w");


	// Init robot localization, Global coordinates
	x_Robot_GLobal_Location = 0.0F;
	y_Robot_GLobal_Location = 0.0F;
	theta_Robot_GLobal_Location = 0.0F;

	// Read goal location, Global coordinates
	x_goal = 4.0F;
	y_goal = 4.0F;

	printf("Testing the ND\n");

	// INIT ND 
	printf(".");
	InitND(&NDparam);
	InicializarND(&NDparam);

	// Run ND
	i=0;
	while (i<2000 && 
		sqrt(pow(x_Robot_GLobal_Location-x_goal,2)+pow(y_Robot_GLobal_Location-y_goal,2))>0.1){
		
		// -----------------------
		// Read Sensors and other Stuff
		// -----------------------

		// Read robot localization, Global coordinates 
		// In reality you eliminate the simulation and here you read odometry
		x_Robot_GLobal_Location = x_Robot_GLobal_Location;
		y_Robot_GLobal_Location = y_Robot_GLobal_Location;
		theta_Robot_GLobal_Location = theta_Robot_GLobal_Location;

		// Read goal location, Global coordinates
		// You only need this if you recalculate the goal (i.e. tracking a person)
		x_goal = x_goal;
		y_goal = y_goal;
	
		// Read sensors (create map), Global coordinates
		x_obstacle1 = 1.0F;
		y_obstacle1 = 0.9F;
		x_obstacle2 = 1.0F;
		y_obstacle2 = 1.4F;
		x_obstacle3 = 1.5F;
		y_obstacle3 = 1.5F;

		// -----------------------
		// Fill variables for the ND
		// -----------------------

		// Goal location
		objetivo.x = x_goal;
		objetivo.y = y_goal;
		
		// Robot location
		movimiento.SR1.posicion.x = x_Robot_GLobal_Location;
		movimiento.SR1.posicion.y = y_Robot_GLobal_Location;
		movimiento.SR1.orientacion = theta_Robot_GLobal_Location;

		// Environment
		entorno.punto[0].x = x_obstacle1;
		entorno.punto[0].y = y_obstacle1;
		entorno.punto[1].x = x_obstacle2;
		entorno.punto[1].y = y_obstacle2;
		entorno.punto[2].x = x_obstacle3;
		entorno.punto[2].y = y_obstacle3;
		entorno.longitud=2;

		// -----------------------
		// Call the ND
		// -----------------------

		velo = IterarND(objetivo,
				    	&movimiento,
					    &entorno,
					    NULL);

		printf("Sol % d -- <v,w,th> = <%2.2f,%2.2f,%2.2f> ",i,velo->v,velo->w,velo->v_theta); 
		i++;

		if (velo->v == 0.0F && velo->w == 0.0F)
			break;

		// -----------------------
		// Simulate motion (to debbug) 
		// -----------------------
		x_Robot_GLobal_Location	= x_Robot_GLobal_Location + velo->v * cos(theta_Robot_GLobal_Location + velo->v_theta) *  NDparam.T;
		y_Robot_GLobal_Location	= y_Robot_GLobal_Location + velo->v * sin(theta_Robot_GLobal_Location + velo->v_theta) *  NDparam.T;
		theta_Robot_GLobal_Location	= theta_Robot_GLobal_Location + velo->w *  NDparam.T;

		if (theta_Robot_GLobal_Location > 6.28)
			theta_Robot_GLobal_Location	= theta_Robot_GLobal_Location - 6.28;
		if (theta_Robot_GLobal_Location < -6.28)
			theta_Robot_GLobal_Location	= 6.28 + theta_Robot_GLobal_Location;


		printf("Pos -- <x,y,th> = <%2.2f,%2.2f,%2.2f> \n",x_Robot_GLobal_Location,
			y_Robot_GLobal_Location,
			theta_Robot_GLobal_Location); 
		
		fprintf(fp,"%f %f %f\n",x_Robot_GLobal_Location,
			y_Robot_GLobal_Location,
			theta_Robot_GLobal_Location);
	}

	printf("end!\n");

	fclose(fp);

}